@@ -1,7 +1,10 @@
 from findpeaks.findpeaks import findpeaks
 
-# from findpeaks.utils.interpolate import interpolate_line1d, interpolate_line2d
-# import findpeaks.utils.stats as stats
+from findpeaks.findpeaks import (
+    import_example,
+    stats,
+    interpolate,
+    )
 
 # Import the denosing filters
 from findpeaks.filters.lee import lee_filter
@@ -15,7 +18,7 @@ from findpeaks.filters.mean import mean_filter
 
 __author__ = 'Erdogan Tasksen'
 __email__ = 'erdogant@gmail.com'
-__version__ = '2.3.4'
+__version__ = '2.4.0'
 
 # module level doc-string
 __doc__ = """
