@@ -6,7 +6,7 @@
 # Licence     : See Licences
 # ----------------------------------------------------
 
-import findpeaks
+# import findpeaks
 from peakdetect import peakdetect
 from caerus import caerus
 import caerus.utils.csplots as csplots
@@ -17,15 +17,18 @@ import numpy as np
 import os
 import requests
 from urllib.parse import urlparse
-import sys
 
-fpath = os.path.join(os.path.dirname(__file__), 'utils')
+# import sys
+# fpath = os.path.join(os.path.dirname(__file__), 'utils')
 # print(fpath)
-if not np.isin(fpath, sys.path): sys.path.append(fpath)
+# if not np.isin(fpath, sys.path): sys.path.append(fpath)
 # sys.path.append(fpath)
-import utils.stats as stats
-import utils.interpolate as interpolate
+import findpeaks.stats as stats
+from findpeaks.stats import disable_tqdm
+import findpeaks.interpolate as interpolate
 
+
+# %%
 class findpeaks():
     """For the detection of peaks in 1d and 2d data.
 
@@ -44,7 +47,7 @@ class findpeaks():
         1d-vector approaches:
             * 'topology'
             * 'peakdetect' (default)
-            * 'cearus'
+            * 'caerus'
         2d-array approaches:
             * 'topology' (default)
             * 'mask'
@@ -77,7 +80,7 @@ class findpeaks():
     cu : float, (default: 0.25)
         The noise variation coefficient, applies for methods: ['kuan','lee','lee_enhanced']
     params : dict() (Default: None)
-        Caerus parameters can be defined in this dict. If None defined, then all default caerus parameters are used:
+        caerus parameters can be defined in this dict. If None defined, then all default caerus parameters are used:
         {'window': 50, 'minperc': 3, 'nlargest': 10, 'threshold': 0.25}
     togray : bool, (default : False)
         Conversion to gray scale.
@@ -124,14 +127,10 @@ class findpeaks():
 
     """
 
-    def __init__(self, method=None, whitelist=['peak','valley'], lookahead=200, interpolate=None, limit=None, imsize=None, scale=True, togray=True, denoise='fastnl', window=3, cu=0.25, params=None, figsize=(15, 8), verbose=3):
+    def __init__(self, method=None, whitelist=['peak','valley'], lookahead=200, interpolate=None, limit=None, imsize=None, scale=True, togray=True, denoise='fastnl', window=3, cu=0.25, params_caerus={'window': 50, 'minperc': 3, 'nlargest': 10, 'threshold': 0.25}, figsize=(15, 8), verbose=3):
         """Initialize findpeaks parameters."""
 
         # Store in object
-        params_caerus = {'window': 50, 'minperc': 3, 'nlargest': 10, 'threshold': 0.25}
-        if params is not None:
-            params_caerus.update(params)
-
         if isinstance(whitelist, str): whitelist=[whitelist]
         if lookahead is None: lookahead=1
         lookahead = np.maximum(1, lookahead)
@@ -147,10 +146,14 @@ class findpeaks():
         self.denoise = denoise
         self.window = window
         self.cu = cu
-        self.params = params_caerus
         self.figsize = figsize
         self.verbose = verbose
 
+        # Store parameters for caerus
+        caerus_defaults = {'window': 50, 'minperc': 3, 'nlargest': 10, 'threshold': 0.25}
+        params_caerus   = {**caerus_defaults, **params_caerus}
+        self.params_caerus = params_caerus
+
     def fit(self, X, x=None):
         """Detect peaks and valleys in a 1D vector or 2D-array (image).
 
@@ -265,13 +268,13 @@ class findpeaks():
             # Post processing for the topology method
             result['topology'] = stats._post_processing(X, Xraw, result['valley'], result['peak'], self.interpolate, 1)
         elif method=='caerus':
-            cs = caerus(**self.params)
-            result = cs.fit(X, return_as_dict=True)
+            cs = caerus(**self.params_caerus)
+            result = cs.fit(X, return_as_dict=True, verbose=self.verbose)
             # Post processing for the caerus method
             result['caerus'] = stats._post_processing(X, Xraw, np.c_[result['loc_start_best'], result['loc_start_best']], np.c_[result['loc_stop_best'], result['loc_stop_best']], self.interpolate, 1, labxRaw=result['df']['labx'].values)
             result['caerus']['model'] = cs
         else:
-            print('[findpeaks] >Method [%s] is not supported in 1d-vector data. <return>' %(self.method))
+            if self.verbose>=2: print('[findpeaks] >WARNING: [method="%s"] is not supported in 1d-vector data. <return>' %(self.method))
             return None
         # Store
         self.results, self.args = self._store1d(X, Xraw, x, result)
@@ -384,7 +387,7 @@ class findpeaks():
         # Arguments
         args = {}
         args['method'] = self.method
-        args['params'] = self.params
+        args['params_caerus'] = self.params_caerus
         args['lookahead'] = self.lookahead
         args['interpolate'] = self.interpolate
         args['figsize'] = self.figsize
@@ -474,7 +477,7 @@ class findpeaks():
             # Compute peaks using local maximum filter.
             result = stats.mask(Xproc, limit=self.limit)
         else:
-            print('[findpeaks] >Method [%s] is not supported in 2d-array (image) data. <return>' %(self.method))
+            if self.verbose>=2: print('[findpeaks] >WARNING: [method="%s"] is not supported in 2d-array (image) data. <return>' %(self.method))
             return None
 
         # Store
@@ -500,7 +503,8 @@ class findpeaks():
             # results['valley'] = result['valley'] # These values are incorrect when using 2d
             results['groups0'] = result['groups0']
         if self.method=='mask':
-            results['Xdetect'] = result
+            results['Xdetect'] = result['Xdetect']
+            results['Xranked'] = result['Xranked']
 
         # Store arguments
         args = {}
@@ -554,7 +558,7 @@ class findpeaks():
 
         # Resize
         if self.imsize:
-            X = stats.resize(X, size=self.imsize)
+            X = stats.resize(X, size=self.imsize, verbose=self.verbose)
             if showfig:
                 # plt.figure(figsize=self.figsize)
                 ax[iax].imshow(X, cmap=('gray_r' if self.togray else None))
@@ -612,7 +616,7 @@ class findpeaks():
 
         """
         if not hasattr(self, 'results'):
-            print('[findpeaks] Nothing to plot.')
+            if self.verbose>=2: print('[findpeaks] >WARNING: Nothing to plot. <return>')
             return None
 
         figsize = figsize if figsize is not None else self.args['figsize']
@@ -623,7 +627,7 @@ class findpeaks():
             # fig_axis = self.plot2d(figsize=figsize)
             fig_axis = self.plot_mask(figsize=figsize, cmap=cmap, text=text)
         else:
-            print('[findpeaks] Nothing to plot for %s' %(self.args['type']))
+            if self.verbose>=2: print('[findpeaks] >WARNING: Nothing to plot for %s' %(self.args['type']))
             return None
 
         # Return
@@ -645,7 +649,7 @@ class findpeaks():
 
         """
         if not self.args['type']=='peaks1d':
-            print('[findpeaks] >Requires results of 1D data <return>.')
+            if self.verbose>=3: print('[findpeaks] >Requires results of 1D data <return>.')
             return None
 
         figsize = figsize if figsize is not None else self.args['figsize']
@@ -689,7 +693,7 @@ class findpeaks():
 
         """
         if not self.args['type']=='peaks2d':
-            print('[findpeaks] >Requires results of 2D data <return>.')
+            if self.verbose>=3: print('[findpeaks] >Requires results of 2D data <return>.')
             return None
         ax_method, ax_mesh = None, None
         figsize = figsize if figsize is not None else self.args['figsize']
@@ -718,7 +722,7 @@ class findpeaks():
 
         """
         if (not hasattr(self, 'results')) or (self.type=='peaks1d'):
-            if self.verbose>=3: print('[findpeaks] >Nothing to plot. Hint: run the fit() function with image data.')
+            if self.verbose>=2: print('[findpeaks] >WARNING: Nothing to plot. Hint: run fit(X), where X is the (image) data. <return>')
             return None
 
         _ = self.preprocessing(X=self.results['Xraw'], showfig=True)
@@ -741,7 +745,7 @@ class findpeaks():
 
         """
         if (self.type=='peaks1d'):
-            if self.verbose>=3: print('[findpeaks] >Nothing to plot. Hint: run the fit() function with 2d-array (image) data.')
+            if self.verbose>=2: print('[findpeaks] >WARNING: Nothing to plot. Hint: run fit(X), where X is the 2d-array (image). <return>')
             return None
 
         if limit is None: limit = self.limit
@@ -779,11 +783,11 @@ class findpeaks():
         ax3.grid(False)
         
         if text:
-            for idx in tqdm(zip(idx_peaks[0], idx_peaks[1])):
+            for idx in tqdm(zip(idx_peaks[0], idx_peaks[1]), disable=disable_tqdm(self.verbose)):
                 ax2.text(idx[1], idx[0], 'p'+self.results['Xranked'][idx].astype(str))
                 ax3.text(idx[1], idx[0], 'p'+self.results['Xranked'][idx].astype(str))
     
-            for idx in tqdm(zip(idx_valleys[0], idx_valleys[1])):
+            for idx in tqdm(zip(idx_valleys[0], idx_valleys[1]), disable=disable_tqdm(self.verbose)):
                 ax2.text(idx[1], idx[0], 'v'+self.results['Xranked'][idx].astype(str))
                 ax3.text(idx[1], idx[0], 'v'+self.results['Xranked'][idx].astype(str))
         
@@ -828,7 +832,7 @@ class findpeaks():
 
         """
         if not hasattr(self, 'results'):
-            if self.verbose>=3: print('[findpeaks] >Nothing to plot. Hint: run the fit() function.')
+            if self.verbose>=2: print('[findpeaks] >WARNING: Nothing to plot. Hint: run the fit() function. <return>')
             return None
         if self.results.get('Xproc', None) is None:
             if self.verbose>=3: print('[findpeaks] >These analysis do not support mesh plotting. This may be caused because your are analysing 1D.')
@@ -912,14 +916,13 @@ class findpeaks():
         """
         if verbose is None: verbose=self.verbose
         if (self.method!='topology') or (not hasattr(self, 'results')):
-            if verbose>=3: print('[findpeaks] >Nothing to plot. Hint: run the .fit(method="topology") function.')
+            if verbose>=3: print('[findpeaks] >WARNING: Nothing to plot. Hint: run the .fit(method="topology") function. <return>')
             return None
 
         # Setup figure
         figsize = figsize if figsize is not None else self.args['figsize']
         fig, (ax1, ax2) = plt.subplots(1, 2, figsize=figsize)
 
-        disable = (True if ((verbose==0 or verbose is None) or verbose>3) else False)
         if self.args['type']=='peaks1d':
             # minpers = 0
             min_peaks = self.results['df']['x'].loc[self.results['df']['valley']].values
@@ -930,7 +933,7 @@ class findpeaks():
             y = self.results['df']['y'].values
             x = self.results['df']['x'].values
             idx = np.where(self.results['df']['rank']>0)[0]
-            for i in tqdm(idx, disable=disable):
+            for i in tqdm(idx, disable=disable_tqdm(verbose)):
                 ax1.text(x[i], (y[i] + y[i] * 0.01), str(self.results['df']['rank'].iloc[i]), color='b')
 
         else:
@@ -942,7 +945,7 @@ class findpeaks():
             # Plot the detected loci
             if verbose>=3: print('[findpeaks] >Plotting loci of birth..')
             ax1.set_title("Loci of births")
-            for i, homclass in tqdm(enumerate(self.results['groups0']), disable=disable):
+            for i, homclass in tqdm(enumerate(self.results['groups0']), disable=disable_tqdm(verbose)):
                 p_birth, bl, pers, p_death = homclass
                 if (self.limit is None):
                     y, x = p_birth
@@ -966,7 +969,7 @@ class findpeaks():
         x = self.results['persistence']['birth_level'].values
         y = self.results['persistence']['death_level'].values
         plt.plot(x, y, '.', c='b')
-        for i in tqdm(range(0,len(x)), disable=disable):
+        for i in tqdm(range(0,len(x)), disable=disable_tqdm(verbose)):
             plt.text(x[i], (y[i] + y[i] * 0.01),  str(i + 1), color='b')
 
         # if verbose>=3: print('[findpeaks] >Plotting Peristence..')
@@ -1020,7 +1023,7 @@ class findpeaks():
             Dataset containing mixed features.
 
         """
-        X = _import_example(data=data, url=url, sep=sep, verbose=self.verbose, datadir=datadir)
+        X = import_example(data=data, url=url, sep=sep, verbose=self.verbose, datadir=datadir)
         return X
 
 
@@ -1049,7 +1052,7 @@ def _plot_original(X, xs, labx, min_peaks, max_peaks, title=None, legend=True, a
 
 
 # %% Import example dataset from github.
-def _import_example(data='2dpeaks', url=None, sep=';', verbose=3, datadir=None):
+def import_example(data='2dpeaks', url=None, sep=';', verbose=3, datadir=None):
     """Import example dataset from github source.
 
     Description
@@ -1094,10 +1097,10 @@ def _import_example(data='2dpeaks', url=None, sep=';', verbose=3, datadir=None):
     elif (data=='btc') or (data=='facebook'):
         from caerus import caerus
         cs = caerus()
-        X = cs.download_example(name=data)
+        X = cs.download_example(name=data, verbose=verbose)
         return X
     else:
-        if verbose>=3: print('[findpeaks] >Nothing to download <return>.')
+        if verbose>=2: print('[findpeaks] >WARNING: Nothing to download. <return>')
         return None
 
     if datadir is None: datadir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')
@@ -1122,3 +1125,9 @@ def _import_example(data='2dpeaks', url=None, sep=';', verbose=3, datadir=None):
         X = pd.read_csv(PATH_TO_DATA, sep=sep).values
     # Return
     return X
+
+
+# %%
+# def disable_tqdm(verbose):
+    # """Set the verbosity messages."""
+    # return  (True if ((verbose==0 or verbose is None) or verbose>3) else False)
